@@ -662,3 +662,12 @@ def test_write_stream(test_images, tmp_path):
     # Note: No assertions here, because video compression is lossy and
     # imageio-python changes the shape of the array. Our PyAV plugin (which
     # should be preferred) does not have the latter limitaiton :)
+
+
+def test_h264_reading(test_images, tmp_path):
+    # regression test for
+    # https://github.com/imageio/imageio/issues/900
+    frames = iio3.imread(test_images / "cockatoo.mp4")
+    iio3.imwrite(tmp_path / "cockatoo.h264", frames, plugin="FFMPEG")
+
+    imageio.get_reader(tmp_path / "cockatoo.h264", "ffmpeg")
